{************************************************************************}
{     Message Master by Daniele Teti - TDSoft@libero.it                  }
{     Version 1.0.4                                                      }
{                                                                        }
{     Italian/English/Russian/Portuguese/German/French/Greek/Spanish/    }
{     Hungarian/Romanian/Finnish                                         }
{       message/input dialogs                                            }
{                                                                        }
{     Delphi Visual Component Library                                    }
{     Copyright (c) 1995,2004 Borland International                      }
{                                                                        }
{     Initial idea and some portions of code by follow people...         }
{                                                                        }
{                                                                        }
{     Tiny portions by Andrew Anoshkin '1997-98                          }
{     Portuguese translation by Renato M. Prado                          }
{     Turkish translation by Egemen EN                                  }
{ German translation by Godfrey McLean  }
{     Dutch translation by Leonard Wennekers                             }
{ French translation by Jean-Fabien Connault  }
{     Greek translation by Babis Michail                                 }
{     Spanish translation by Jorge R. Lopez Danieluk                     }
{     Italian translation by Daniele Teti                                }
{     Hungarian and Romanian translation by Zoltan Zorgo                 }
{                                                                        }
{     Many thanks to Eugene Genev for his XiComponents                   }
{
      History:
      02/12/2004 Initial Release
      13/12/2004 Added ShowError Procedure
      20/12/2004 Some bug fix
      21/01/2005 Published on http://dade2000.altervista.org
      04/02/2005 Version 1.0.3
                 Localization data changed to "array"
                 Language names cross reference added (needs review for unicode language)
                 Demo updated
                 Thanks to Zoltan Zorgo
      04/11/2005 Version 1.0.4
                 Delphi 2005 Compatibility
                 Added "Confirm" Function
                 Added "IsStatusShowed" Function
                 Added "ScreenCenter" Function
      --/--/2014 Added Lazarus (Win32/64) Compatibility
                 Added Finnish language
}
{************************************************************************}

unit MessDlgs;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

{$IFDEF Win32}{$DEFINE Windows}{$ENDIF}
{$IFDEF Win64}{$DEFINE Windows}{$ENDIF}
{$IFDEF FPC}{$DEFINE DELPHI12_UP}{$ENDIF}
{$IFDEF VER200}{$DEFINE DELPHI12_UP}{$ENDIF}
{$IFDEF VER210}{$DEFINE DELPHI12_UP}{$ENDIF}
{$IFDEF VER220}{$DEFINE DELPHI12_UP}{$ENDIF}
{$IFDEF VER230}{$DEFINE DELPHI12_UP}{$ENDIF}
{$IFDEF VER240}{$DEFINE DELPHI12_UP}{$ENDIF}
{$IFDEF VER250}{$DEFINE DELPHI12_UP}{$ENDIF}

uses
{$IFNDEF FPC}
  Consts,
{$ENDIF}
{$IFDEF Windows}
  Windows,
{$ELSE}
  LCLIntf, LCLType,
{$ENDIF}
  SysUtils, Classes, Controls, Graphics, Dialogs, Forms,
  StdCtrls, ExtCtrls, XiProgressBar, XiPanel, XiButton;

{ Message/Input dialogs }

{$I International.inc} {TLangType type definition moved into "International.inc"}

{$IFNDEF Windows}{$R MSGDL.RES}{$ENDIF}

{$IFNDEF Windows}
type
TNonClientMetrics = packed record
  cbSize: UINT;
  iBorderWidth: Integer;
  iScrollWidth: Integer;
  iScrollHeight: Integer;
  iCaptionWidth: Integer;
  iCaptionHeight: Integer;
  lfCaptionFont: TLogFontA;
  iSmCaptionWidth: Integer;
  iSmCaptionHeight: Integer;
  lfSmCaptionFont: TLogFontA;
  iMenuWidth: Integer;
  iMenuHeight: Integer;
  lfMenuFont: TLogFontA;
  lfStatusFont: TLogFontA;
  lfMessageFont: TLogFontA;
  end;
{$ENDIF}

type
  TProgressColorScheme = (procsCustom, procsDesert, procsGrass, procsSilver,
    procsSky, procsRose, procsSun,
    procsHackers, procsNight, procsVelvet, procsMetal, procsViolet, procsToxic);
  TPanelColorScheme = (pnlcsCustom, pnlcsDesert, pnlcsGrass, pnlcsSilver,
    pnlcsSky, pnlcsRose, pnlcsSun);

  TButtonColorScheme = (btncsNeoDesert, btncsNeoSky, btncsNeoGrass,
    btncsNeoSilver, btncsNeoRose, btncsNeoSun, btncsDesert, btncsGrass,
    btncsSky, btncsSun, btncsRose, btncsSilver, btncsCustom);

  TPercentage = 1..100;

  TMsgOptions = record
    {Prima di visualizzare la finestra di status controlla che l'applicazione
    abbia il focus e che sia quella attiva}
    SilentStatus: Boolean;
    {Prima di visualizzare la finestra di status controlla che l'applicazione
    abbia il focus e che la form visualizzata abbia questo handle}
    ShowStatusOnlyOnForm: THandle;

    //Lingua
    DefLang: TLangType;
    UseGradient: Boolean; //Abilita il gradient

    {------------------------------}
    UseBorder: boolean; // = True;
    UseCustomButtons: Boolean; //XiButtons
    CustomButtonsColorScheme: TButtonColorScheme; //XiButtons Color Scheme
    CustomPanelColorScheme: TPanelColorScheme; //XiPanel Color Scheme
    CustomProgressColorScheme: TProgressColorScheme; //XiProgress Color Scheme
    StatusHeight, StatusWidth: Cardinal;
    UseCustomPanel: Boolean;
    {------------------------------}

    UseCustomFont: Boolean; //Abilita il font personalizzato
    UseShapedForm: boolean; //Abilita l'usa di form con bordi stondati
    FormRoundSize: Cardinal; //Raggio dell'angolo delle form
    StartGradientColor, EndGradientColor: TColor;
    //Colore di inizio e di fine gradient
    Font: TFont; //Font usato (nal caso si CustomFont abilitato)
  end;

var
  MsgOptions: TMsgOptions;

procedure DrawGradient(ACanvas: TCanvas; Rect: TRect;
  Horizontal: Boolean; Colors: array of TColor);

procedure ShowMessage(const Message: string);
procedure ShowError(const Message: string);

function Confirm(Message: string = 'Are you sure?'): boolean;

function ScreenCenter: TPoint;

function MessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint):
  Integer;

function MessageDlgI(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; Lang: TLangType): Integer;
function MessageDlgPosI(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer; Lang: TLangType):
  Integer;

function InputQueryI(const ACaption, APrompt: string;
  var Value: string; Lang: TLangType): Boolean;

function InputQuery(const ACaption, APrompt: string;
  var Value: string): Boolean;

function InputBoxI(const ACaption, APrompt, ADefault: string; Lang: TLangType):
  string;
function InputBox(const ACaption, APrompt, ADefault: string):
  string;

function CreateMessageDialogI(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Lang: TLangType): TForm;

{-------- Status ---------------------}
function IsStatusShowed: Boolean;
procedure ShowStatus(const sCaption: string; ProgressVisible: boolean = True);
procedure ShowStatusPos(const sCaption: string; ProgressVisible: boolean = true;
  ShowInCenter: boolean = true); overload;
procedure ShowStatusPos(const sCaption: string; sPosition: TPoint;
  ProgressVisible: boolean = true); overload;
procedure UpdateStatus(const Perc: TPercentage); overload;
procedure UpdateStatus(const sCaption: string); overload;
procedure UpdateStatus(const Perc: TPercentage; const sCaption: string);
overload;
procedure CloseStatus;

const
  VERSION = '1.0.4';

implementation

Uses Types;

var
  FormRegion: HRGN;
  StatusForm: TForm = nil;
  StatusLabel: TLabel;
  StatusProgress: TXiProgressBar;

procedure DrawGradient(ACanvas: TCanvas; Rect: TRect;
  Horizontal: Boolean; Colors: array of TColor);
type
  RGBArray = array[0..2] of Byte;
var
  x, y, z, stelle, mx, bis, faColorsh, mass: Integer;
  Faktor: double;
  A: RGBArray;
  B: array of RGBArray;
  merkw: integer;
  merks: TPenStyle;
  merkp: TColor;
begin
  mx := High(Colors);
  if mx > 0 then
  begin
    if Horizontal then
      mass := Rect.Right - Rect.Left
    else
      mass := Rect.Bottom - Rect.Top;
    SetLength(b, mx + 1);
    for x := 0 to mx do
    begin
      Colors[x] := ColorToRGB(Colors[x]);
      b[x][0] := GetRValue(Colors[x]);
      b[x][1] := GetGValue(Colors[x]);
      b[x][2] := GetBValue(Colors[x]);
    end;
    merkw := ACanvas.Pen.Width;
    merks := ACanvas.Pen.Style;
    merkp := ACanvas.Pen.Color;
    ACanvas.Pen.Width := 1;
    ACanvas.Pen.Style := psSolid;
    faColorsh := Round(mass / mx);
    for y := 0 to mx - 1 do
    begin
      if y = mx - 1 then
        bis := mass - y * faColorsh - 1
      else
        bis := faColorsh;
      for x := 0 to bis do
      begin
        Stelle := x + y * faColorsh;
        faktor := x / bis;
        for z := 0 to 3 do
          a[z] := Trunc(b[y][z] + ((b[y + 1][z] - b[y][z]) * Faktor));
        ACanvas.Pen.Color := RGB(a[0], a[1], a[2]);
        if Horizontal then
        begin
          ACanvas.MoveTo(Rect.Left + Stelle, Rect.Top);
          ACanvas.LineTo(Rect.Left + Stelle, Rect.Bottom);
        end
        else
        begin
          ACanvas.MoveTo(Rect.Left, Rect.Top + Stelle);
          ACanvas.LineTo(Rect.Right, Rect.Top + Stelle);
        end;
      end;
    end;
    b := nil;
    ACanvas.Pen.Width := merkw;
    ACanvas.Pen.Style := merks;
    ACanvas.Pen.Color := merkp;
  end
  else
    // Please specify at least two colors
    raise
      EMathError.Create('Please specify at least two colors');
end;

function Max(I, J: Integer): Integer;
begin
  if I > J then
    Result := I
  else
    Result := J;
end;

function GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do
    Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do
    Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;

type
  TMessageForm = class(TForm)
  private
    procedure HelpButtonClick(Sender: TObject);
  public
    procedure PaintGradient(Sender: TObject);
    constructor CreateNew(AOwner: TComponent); reintroduce;
    destructor Destroy; override;
  end;

constructor TMessageForm.CreateNew(AOwner: TComponent);
var
  NonClientMetrics: TNonClientMetrics;
begin
  inherited CreateNew(AOwner);

  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    Font.Handle := CreateFontIndirect(NonClientMetrics.lfMessageFont);

  if MsgOptions.UseGradient and (not MsgOptions.UseCustomPanel) then
    OnPaint := PaintGradient;

  if MsgOptions.UseCustomFont then
    Font.Assign(MsgOptions.Font);
end;

destructor TMessageForm.Destroy;
begin
  DeleteObject(FormRegion);
  inherited;
end;

procedure TMessageForm.HelpButtonClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

function CreateMessageDialogI(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; Lang: TLangType): TForm;
const
  mcHorzMargin = 8;
  mcVertMargin = 8;
  mcHorzSpacing = 10;
  mcVertSpacing = 10;
  mcButtonWidth = 50;
  mcButtonHeight = 14;
  mcButtonSpacing = 4;

const
  {$IFDEF Windows}
  IconIDs: array[TMsgDlgType] of PChar = (IDI_EXCLAMATION, IDI_HAND,
    IDI_ASTERISK, IDI_QUESTION, nil);
  {$ELSE}
  IconIDs: array[TMsgDlgType] of PChar = ('IDI_EXCLAMATION', 'IDI_HAND',
    'IDI_ASTERISK', 'IDI_QUESTION', nil);{$ENDIF}

  ButtonNames: array[TMsgDlgBtn] of string = (
    'Yes', 'No', 'OK', 'Cancel', 'Abort', 'Retry', 'Ignore', 'All', 'NoToAll',
    'YesToAll', 'Help'{$IFDEF DELPHI12_UP},'Close'{$ENDIF});
  ModalResults: array[TMsgDlgBtn] of Integer = (
    mrYes, mrNo, mrOk, mrCancel, mrAbort, mrRetry, mrIgnore, mrAll, mrNoToAll,
    mrYesToAll, 0{$IFDEF DELPHI12_UP},0{$ENDIF});

var
  DialogUnits: TPoint;
  HorzMargin, VertMargin, HorzSpacing, VertSpacing, ButtonWidth,
    ButtonHeight, ButtonSpacing, ButtonCount, ButtonGroupWidth,
    IconTextWidth, IconTextHeight, X: Integer;
  B, DefaultButton, CancelButton: TMsgDlgBtn;
  IconID: PChar;
  TextRect: TRect;
  TheParent: TWinControl;
  pnl: TXiPanel;
begin
  Result := TMessageForm.CreateNew(Application);
  with Result do
  begin
{$IFDEF VER170}
    PopupMode := pmAuto;
{$ENDIF}

    if MsgOptions.UseCustomFont then
      Font.Assign(MsgOptions.Font);
    if MsgOptions.UseCustomPanel then
    begin
      pnl := TXiPanel.Create(result);
      with pnl do
      begin
        Name := 'ThePanel';
        Caption := '';
        Align := alClient;
        ColorScheme :=
          XiPanel.TColorScheme(MsgOptions.CustomPanelColorScheme);
        Parent := result;
      end;
      TheParent := pnl;
    end
    else
      TheParent := Result;

    if MsgOptions.UseBorder then
      BorderStyle := bsDialog
    else
      BorderStyle := bsNone;

    Canvas.Font.Assign(Font);

    if MsgOptions.UseCustomPanel then
      Canvas.Font.Color := pnl.ColorDark;

    DialogUnits := GetAveCharSize(Canvas);
    HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
    VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
    HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
    VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
    ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4);
    ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
    ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4);
    SetRect(TextRect, 0, 0, Screen.Width div 2, 0);
    DrawText(Canvas.Handle, PChar(Msg), -1, TextRect,
      DT_CALCRECT or DT_WORDBREAK);
    IconID := IconIDs[DlgType];
    IconTextWidth := TextRect.Right;
    IconTextHeight := TextRect.Bottom;
    if IconID <> nil then
    begin
      Inc(IconTextWidth, 32 + HorzSpacing);
      if IconTextHeight < 32 then
        IconTextHeight := 32;
    end;
    ButtonCount := 0;
    for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
      if B in Buttons then
        Inc(ButtonCount);
    ButtonGroupWidth := 0;
    if ButtonCount <> 0 then
      ButtonGroupWidth := ButtonWidth * ButtonCount +
        ButtonSpacing * (ButtonCount - 1);
    ClientWidth := Max(IconTextWidth, ButtonGroupWidth) + HorzMargin * 2;
    ClientHeight := IconTextHeight + ButtonHeight + VertSpacing +
      VertMargin * 2;
    Left := (Screen.Width div 2) - (Width div 2);
    Top := (Screen.Height div 2) - (Height div 2);

    if DlgType <> mtCustom then
      //Support International
     {$IFDEF FPC}Caption := UTF8Encode(Internationals[Lang].Captions[DlgType])
     {$ELSE}Caption := Internationals[Lang].Captions[DlgType]{$ENDIF}  
    else
      Caption := Application.Title;
    if IconID <> nil then
      with TImage.Create(TheParent) do
      begin
        Name := 'Image';
        Parent := TheParent;
        {$IFDEF Windows}Picture.Icon.Handle := Windows.LoadIcon(0, IconID);
        {$ELSE}         Picture.Icon.Handle := {LCLIntf.}LoadIcon(hinstance, IconID);{$ENDIF}
        SetBounds(HorzMargin, VertMargin, 32, 32);
      end;
    with TLabel.Create(TheParent) do
    begin
      //Color:=clRed;
      Autosize := True; //?
      Transparent := True;
      //Transparent := false;

      Name := 'Message';
      Parent := TheParent;
      WordWrap := True;
      Caption := Msg;
      Font.Assign(MsgOptions.Font);
      if MsgOptions.UseCustomPanel then
        Font.Color := pnl.ColorDark;

      BoundsRect := TextRect;
      SetBounds(IconTextWidth - TextRect.Right + HorzMargin, VertMargin,
        TextRect.Right, TextRect.Bottom * 2);
      BringToFront;
    end;
    if mbOk in Buttons then
      DefaultButton := mbOk
    else if mbYes in Buttons then
      DefaultButton := mbYes
    else
      DefaultButton := mbRetry;
    if mbCancel in Buttons then
      CancelButton := mbCancel
    else if mbNo in Buttons then
      CancelButton := mbNo
    else
      CancelButton := mbOk;
    X := (ClientWidth - ButtonGroupWidth) div 2;
    for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
      if B in Buttons then
      begin
        if MsgOptions.UseCustomButtons then
        begin
          with TXiButton.Create(Result) do
          begin
            ColorScheme :=
              XiButton.TColorScheme(MsgOptions.CustomButtonsColorScheme);
            Name := ButtonNames[B];
            Parent := TheParent;

            //Support International
            {$IFDEF FPC}Caption := UTF8Encode(Internationals[Lang].ButtonCaptions[B]);
            {$ELSE}Caption := Internationals[Lang].ButtonCaptions[B];{$ENDIF}
            ModalResult := ModalResults[B];
            if B = DefaultButton then
              Default := True;
            if B = CancelButton then
              Cancel := True;
            SetBounds(X, IconTextHeight + VertMargin + VertSpacing,
              ButtonWidth, ButtonHeight);
            Inc(X, ButtonWidth + ButtonSpacing);
            if B = mbHelp then
              OnClick := TMessageForm(Result).HelpButtonClick;
          end; //End With
        end
        else
        begin
          with TButton.Create(Result) do
          begin
            Name := ButtonNames[B];
            Parent := TheParent;

            //Support International
            {$IFDEF FPC}Caption := UTF8Encode(Internationals[Lang].ButtonCaptions[B]);
            {$ELSE}Caption := Internationals[Lang].ButtonCaptions[B];{$ENDIF}
            ModalResult := ModalResults[B];
            if B = DefaultButton then
              Default := True;
            if B = CancelButton then
              Cancel := True;
            SetBounds(X, IconTextHeight + VertMargin + VertSpacing,
              ButtonWidth, ButtonHeight);
            Inc(X, ButtonWidth + ButtonSpacing);
            if B = mbHelp then
              OnClick := TMessageForm(Result).HelpButtonClick;
          end; //End With
        end;
      end;

    if MsgOptions.UseShapedForm and (not MsgOptions.UseCustomPanel) then
    begin
      BorderStyle := bsNone;
      if MsgOptions.UseBorder then
        Height := Height - 35; //Troppo spazio sotto i bottoni
      FormRegion := CreateRoundRectRgn(0, 0, Width, Height,
        MsgOptions.FormRoundSize,
        MsgOptions.FormRoundSize);
      SetWindowRgn(Handle, FormRegion, True);
    end;
  end;
end;

function MessageDlgI(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; Lang: TLangType): Integer;
begin
  Result := MessageDlgPosI(Msg, DlgType, Buttons, HelpCtx, -1, -1, Lang);
end;

function MessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint):
  Integer;
begin
  Result := MessageDlgI(Msg, DlgType, Buttons, HelpCtx, MsgOptions.DefLang);
end;

function MessageDlgPosI(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer; Lang: TLangType):
  Integer;
begin
  with CreateMessageDialogI(Msg, DlgType, Buttons, Lang) do
  try
    HelpContext := HelpCtx;
    if X >= 0 then
      Left := X;
    if Y >= 0 then
      Top := Y;
    Result := ShowModal;
  finally
    Free;
  end;
end;

{ Input dialog }

function InputQueryI(const ACaption, APrompt: string;
  var Value: string; Lang: TLangType): Boolean;
var
  Form: TForm;
  Prompt: TLabel;
  Edit: TEdit;
  DialogUnits: TPoint;
  ButtonTop, ButtonWidth, ButtonHeight: Integer;
  CancelCaption: string;
  pnl: TXiPanel;
  TheParent: TWinControl;
begin
  Result := False;
  //  Form := TForm.Create(Application);

  Form := TMessageForm.CreateNew(Application);
  with Form do
  try
    Canvas.Font := Font;
    DialogUnits := GetAveCharSize(Canvas);
    if MsgOptions.UseBorder then
      BorderStyle := bsDialog
    else
      BorderStyle := bsNone;

    Caption := ACaption;
    ClientWidth := MulDiv(180, DialogUnits.X, 4);
    ClientHeight := MulDiv(63, DialogUnits.Y, 8);
    Position := poScreenCenter;

    if MsgOptions.UseCustomPanel then
    begin
      pnl := TXiPanel.Create(Form);
      with pnl do
      begin
        Align := alClient;
        ColorScheme :=
          XiPanel.TColorScheme(MsgOptions.CustomPanelColorScheme);
        Parent := Form;
      end;
      TheParent := pnl;
    end
    else
      TheParent := Form;

    Prompt := TLabel.Create(Form);
    with Prompt do
    begin
      Parent := TheParent;
      AutoSize := True;
      Left := MulDiv(8, DialogUnits.X, 4);
      Top := MulDiv(8, DialogUnits.Y, 8);
      Caption := APrompt;
      Transparent := True;
    end;
    Edit := TEdit.Create(Form);
    with Edit do
    begin
      Parent := TheParent;
      Left := Prompt.Left;
      Top := MulDiv(19, DialogUnits.Y, 8);
      Width := MulDiv(164, DialogUnits.X, 4);
      MaxLength := 255;
      Text := Value;
      SelectAll;
    end;
    ButtonTop := MulDiv(41, DialogUnits.Y, 8);
    ButtonWidth := MulDiv(50, DialogUnits.X, 4);
    ButtonHeight := MulDiv(14, DialogUnits.Y, 8);

    ////////// OK Button on InputQuery // Start
    if MsgOptions.UseCustomButtons then
      with TXiButton.Create(Form) do
      begin
        Parent := TheParent;
        Caption := 'OK';
        ModalResult := mrOk;
        Default := True;
        SetBounds(MulDiv(38, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
        ColorScheme :=
          XiButton.TColorScheme(MsgOptions.CustomButtonsColorScheme);
      end
    else
      with TButton.Create(Form) do
      begin
        Parent := TheParent;
        Caption := 'OK';
        ModalResult := mrOk;
        Default := True;
        SetBounds(MulDiv(38, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
      end;
    ////////// OK Button on InputQuery // End

    ////// International support for Cancel Caption  
    {$IFDEF FPC}CancelCaption := UTF8Encode(Internationals[Lang].sCancelCaption);
    {$ELSE}CancelCaption := Internationals[Lang].sCancelCaption;{$ENDIF}

    ////
    if MsgOptions.UseCustomButtons then
      with TXiButton.Create(Form) do
      begin
        Parent := TheParent;
        ModalResult := mrCancel;
        Cancel := True;
        SetBounds(MulDiv(92, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
        ColorScheme :=
          XiButton.TColorScheme(MsgOptions.CustomButtonsColorScheme);
        Caption := CancelCaption;
      end
    else
      with TButton.Create(Form) do
      begin
        Parent := TheParent;
        ModalResult := mrCancel;
        Cancel := True;
        SetBounds(MulDiv(92, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
        Caption := CancelCaption;
      end;

    if MsgOptions.UseShapedForm and (not MsgOptions.UseCustomPanel) then
    begin
      BorderStyle := bsNone;
      if MsgOptions.UseBorder then
        Height := Height - 35; //Troppo spazio sotto i bottoni
      FormRegion := CreateRoundRectRgn(0, 0, Width, Height,
        MsgOptions.FormRoundSize,
        MsgOptions.FormRoundSize);
      SetWindowRgn(Handle, FormRegion, True);
    end;

    if ShowModal = mrOk then
    begin
      Value := Edit.Text;
      Result := True;
    end;
  finally
    Form.Free;
  end;
end;

function InputQuery(const ACaption, APrompt: string;
  var Value: string): Boolean;
begin
  result := InputQueryI(ACaption, APrompt, VAlue, MsgOptions.DefLang);
end;

function InputBoxI(const ACaption, APrompt, ADefault: string; Lang: TLangType):
  string;
begin
  Result := ADefault;
  InputQueryI(ACaption, APrompt, Result, Lang);
end;

function InputBox(const ACaption, APrompt, ADefault: string): string;
begin
  Result := InputBoxI(ACaption, APrompt, ADefault, MsgOptions.DefLang);
end;

procedure ShowMessage(const Message: string);
begin
  MessageDlg(Message, mtInformation, [mbOK], 0);
end;

procedure ShowError(const Message: string);
begin
  MessageDlg(Message, mtError, [mbOK], 0);
end;

procedure TMessageForm.PaintGradient(Sender: TObject);
begin
  DrawGradient(self.Canvas, ClientRect, false, [MsgOptions.StartGradientColor,
    MsgOptions.EndGradientColor]);
end;

{------------- Status ---------}

function IsStatusShowed: Boolean;
begin
  Result := StatusForm <> nil;
end;

procedure CheckStatus(StatusMustBeCreated: boolean);
begin
  if StatusMustBeCreated and (StatusForm = nil) then
    raise Exception.Create('Status not created');
  if (not StatusMustBeCreated) and (StatusForm <> nil) then
    raise Exception.Create('Status already created');
end;

procedure ShowStatusPos(const sCaption: string; ProgressVisible: boolean = true;
  ShowInCenter: boolean = true);
var
  p: TPoint;
begin
  p := CenterPoint(Screen.WorkAreaRect);
  p.X := p.x - MsgOptions.StatusWidth div 2;
  p.Y := p.y - MsgOptions.StatusHeight div 2;
  ShowStatusPos(sCaption, P, ProgressVisible);
end;

procedure ShowStatusPos(const sCaption: string; sPosition: TPoint;
  ProgressVisible: boolean = true);
var
  Panel: TXiPanel;
begin
  CheckStatus(False);
  StatusForm := TMessageForm.CreateNew(Application);
  with StatusForm do
  begin
    Position := poDesigned;
{$IFDEF VER170}
    PopupMode := pmAuto;
{$ENDIF}
    Left := sPosition.x;
    Top := sPosition.y;
    FormStyle := fsStayOnTop;
    BorderStyle := bsNone;
    BorderIcons := [];
    Canvas.Font := Font;
    Width := MsgOptions.StatusWidth;
    Height := MsgOptions.StatusHeight;
  end;

  Panel := TXiPanel.Create(StatusForm);
  with Panel do
  begin
    Parent := StatusForm;
    Align := alClient;
    ColorScheme := XiPanel.TColorScheme(MsgOptions.CustomPanelColorScheme);
    Caption := '';
  end;

  StatusProgress := TXiProgressBar.Create(StatusForm);
  with StatusProgress do
  begin
    Visible := ProgressVisible;
    Top := Trunc(StatusForm.Height * 0.75);
    Left := Trunc(StatusForm.WIdth * 0.05);
    Width := Trunc(StatusForm.WIdth * 0.90);
    Parent := Panel;
    ColorScheme :=
      XiProgressBar.TColorScheme(MsgOptions.CustomProgressColorScheme);
  end;
  StatusLabel := TLabel.Create(StatusForm);
  with StatusLabel do
  begin
    AutoSize := False;
    Parent := Panel;
    Font.Assign(MsgOptions.Font);
    Font.Color := Panel.ColorDark; //Colore dell'ombra del pannello
    Font.Style := [fsBold];
    WordWrap := False;
    Top := Trunc(StatusForm.Height * 0.1);
    Left := Trunc(StatusForm.WIdth * 0.05);
    Width := StatusForm.WIdth - StatusLabel.Left * 2;
    Transparent := True;
    Caption := sCaption; //  + slinebreak + sCaption;
  end;

  StatusForm.Height := StatusLabel.Height + MsgOptions.StatusHeight;

  if MsgOptions.SilentStatus and not Application.Active then
    StatusForm.Visible := False
  else if (MsgOptions.ShowStatusOnlyOnForm = 0) or (GetForegroundWindow =
    MsgOptions.ShowStatusOnlyOnForm) then
  begin
    StatusForm.Visible := True;
    StatusForm.Update;
  end;
end;

procedure ShowStatus(const sCaption: string; ProgressVisible: boolean = True);
var
  p: TPoint;
begin
  p := CenterPoint(Screen.DesktopRect);
  p.X := p.x - (MsgOptions.StatusWidth div 2);
  ShowStatusPos(sCaption, p, ProgressVisible);
end;

procedure UpdateStatus(const Perc: TPercentage);
begin
  CheckStatus(True);
  StatusProgress.Position := Perc;
  StatusForm.Update;
end;

procedure UpdateStatus(const sCaption: string);
begin
  CheckStatus(True);
  StatusLabel.Caption := sCaption;
  StatusForm.Update;
end;

procedure UpdateStatus(const Perc: TPercentage; const sCaption: string);
begin
  UpdateStatus(Perc);
  UpdateStatus(sCaption);
end;

procedure CloseStatus;
begin
  CheckStatus(True);
  FreeAndNil(StatusForm);
end;

function Confirm(Message: string = 'Are you sure?'): boolean;
begin
  Result := MessageDlg(Message, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end;

function ScreenCenter: TPoint;
begin
  Result.X := Screen.Width div 2;
  Result.Y := Screen.Height div 2;
end;

initialization

  with MsgOptions do
  begin
    DefLang := ltEnglish;
    SilentStatus := False;
    ShowStatusOnlyOnForm := 0;
    UseBorder := True;
    UseGradient := False;
    UseCustomButtons := False;
    CustomButtonsColorScheme := btncsSky;
    CustomPanelColorScheme := pnlcsSky;
    CustomProgressColorScheme := procsSky;
    StatusWidth := 300;
    StatusHeight := 70;
    UseCustomFont := False;
    StartGradientColor := 15448477; //clGradientInactiveCaption;
    EndGradientColor := 16749885; //clGradientActiveCaption;
    UseShapedForm := False;
    FormRoundSize := 15;
    Font := TFont.Create;
    with Font do
    begin
      Name := 'Verdana';
      Size := 10;
      Style := [fsBold];
    end;
  end;

finalization
  MsgOptions.Font.Free;

end.

